@import "sphinx-book-theme.css";

/* for the sphinx design cards */
body {
    --sd-color-shadow: dimgrey;
}

dl.citation > dt {
    float: left;
    margin-right: 15px;
    font-weight: bold;
}

/* for custom small role */
.small {
    font-size: 40% !important;
}

.smaller,
.pr {
    font-size: 70% !important;
}

/* Fix pydata theme search button changing size */
.search-button-field {
    border: 0;
    outline: 1px solid var(--pst-color-border);
}
.search-button-field:hover {
    border: 0;
    outline: 2px solid var(--pst-color-link-hover);
}

/* Make the readthedocs search popup adapt to the theme.
 * Might break again soon: https://github.com/readthedocs/addons/issues/570
 */
:root {
    --readthedocs-search-backdrop-color: var(--pst-color-shadow);
    --readthedocs-search-color: var(--pst-color-text-base);
    --readthedocs-search-content-background-color: var(--pst-color-background);
    --readthedocs-search-content-border-color: var(--pst-color-border);
    --readthedocs-search-filters-border-color: var(--pst-color-border);
    --readthedocs-search-font-family: var(--pst-font-family-base);
    --readthedocs-search-font-size: var(--pst-font-size-base);
    --readthedocs-search-footer-background-color: var(--pst-color-on-background);
    --readthedocs-search-footer-code-background-color: var(--pst-color-surface);
    --readthedocs-search-footer-code-border-color: var(--pst-color-border);
    --readthedocs-search-input-background-color: var(--pst-color-surface);
    --readthedocs-search-result-section-border-color: var(--pst-color-border);
    --readthedocs-search-result-section-color: var(--pst-color-link);
    --readthedocs-search-result-section-highlight-color: var(--pst-color-accent);
    --readthedocs-search-result-section-subheading-color: var(--pst-color-text-muted);
}
